@@ -24,18 +24,19 @@
 # Unless otherwise stated in the comments, "id", in e.g. user_id, refers to the
 # internal Keycloak server ID, usually a uuid string
 
-from .urls_patterns import *
-
-from .keycloak_openid import KeycloakOpenID
-
-from .exceptions import raise_error_from_response, KeycloakGetError
-
-from .urls_patterns import (
-    URL_ADMIN_USERS,
-)
+import json
 
 from .connection import ConnectionManager
-import json
+from .exceptions import raise_error_from_response, KeycloakGetError
+from .keycloak_openid import KeycloakOpenID
+from .urls_patterns import URL_ADMIN_SERVER_INFO, URL_ADMIN_CLIENT_AUTHZ_RESOURCES, URL_ADMIN_CLIENT_ROLES, \
+    URL_ADMIN_GET_SESSIONS, URL_ADMIN_RESET_PASSWORD, URL_ADMIN_SEND_UPDATE_ACCOUNT, \
+    URL_ADMIN_USER_CLIENT_ROLES_COMPOSITE, URL_ADMIN_USER_GROUP, URL_ADMIN_REALM_ROLES, URL_ADMIN_GROUP_CHILD, \
+    URL_ADMIN_USER_CONSENTS, URL_ADMIN_SEND_VERIFY_EMAIL, URL_ADMIN_CLIENT, URL_ADMIN_USER, URL_ADMIN_CLIENT_ROLE, \
+    URL_ADMIN_USER_GROUPS, URL_ADMIN_CLIENTS, URL_ADMIN_FLOWS_EXECUTIONS, URL_ADMIN_GROUPS, URL_ADMIN_USER_CLIENT_ROLES, \
+    URL_ADMIN_REALM_IMPORT, URL_ADMIN_USERS_COUNT, URL_ADMIN_FLOWS, URL_ADMIN_GROUP, URL_ADMIN_CLIENT_AUTHZ_SETTINGS, \
+    URL_ADMIN_GROUP_MEMBERS, URL_ADMIN_USER_STORAGE, URL_ADMIN_GROUP_PERMISSIONS, URL_ADMIN_IDPS, \
+    URL_ADMIN_USER_CLIENT_ROLES_AVAILABLE, URL_ADMIN_USERS
 
 
 class KeycloakAdmin:
@@ -114,6 +115,22 @@ class KeycloakAdmin:
     def token(self, value):
         self._token = value
 
+    def import_realm(self, payload):
+        """
+        Import a new realm from a RealmRepresentation. Realm name must be unique.
+
+        RealmRepresentation
+        https://www.keycloak.org/docs-api/4.4/rest-api/index.html#_realmrepresentation
+
+        :param payload: RealmRepresentation
+
+        :return: RealmRepresentation
+        """
+
+        data_raw = self.connection.raw_post(URL_ADMIN_REALM_IMPORT,
+                                            data=json.dumps(payload))
+        return raise_error_from_response(data_raw, KeycloakGetError, expected_code=201)
+
     def get_users(self, query=None):
         """
         Get users Returns a list of users, filtered according to query parameters
@@ -375,7 +392,7 @@ class KeycloakAdmin:
             if subgroup['path'] == path:
                 return subgroup
             elif subgroup["subGroups"]:
-                 for subgroup in group["subGroups"]:
+                for subgroup in group["subGroups"]:
                     return self.get_subgroups(subgroup, path)
         return None
 
@@ -413,9 +430,12 @@ class KeycloakAdmin:
             if group['path'] == path:
                 return group
             elif search_in_subgroups and group["subGroups"]:
-                res = self.get_subgroups(group, path)
-                if res != None:
-                    return res
+                for group in group["subGroups"]:
+                    if group['path'] == path:
+                        return group
+                    res = self.get_subgroups(group, path)
+                    if res != None:
+                        return res
         return None
 
     def create_group(self, payload, parent=None, skip_exists=False):
@@ -435,7 +455,7 @@ class KeycloakAdmin:
         exists = None
 
         if name is None and path is not None:
-          path="/" + name
+            path = "/" + name
 
         elif path is not None:
             exists = self.get_group_by_path(path=path, search_in_subgroups=True)
@@ -444,13 +464,13 @@ class KeycloakAdmin:
             return str(exists)
 
         if parent is None:
-          params_path = {"realm-name": self.realm_name}
-          data_raw = self.connection.raw_post(URL_ADMIN_GROUPS.format(**params_path),
-                                            data=json.dumps(payload))
+            params_path = {"realm-name": self.realm_name}
+            data_raw = self.connection.raw_post(URL_ADMIN_GROUPS.format(**params_path),
+                                                data=json.dumps(payload))
         else:
-          params_path = {"realm-name": self.realm_name, "id": parent,}
-          data_raw = self.connection.raw_post(URL_ADMIN_GROUP_CHILD.format(**params_path),
-                                            data=json.dumps(payload))
+            params_path = {"realm-name": self.realm_name, "id": parent, }
+            data_raw = self.connection.raw_post(URL_ADMIN_GROUP_CHILD.format(**params_path),
+                                                data=json.dumps(payload))
         return raise_error_from_response(data_raw, KeycloakGetError, expected_code=201, skip_exists=skip_exists)
 
     def group_set_permissions(self, group_id, enabled=True):
@@ -825,7 +845,7 @@ class KeycloakAdmin:
 
         params_path = {"realm-name": self.realm_name, "flow-alias": flow_alias}
         data_raw = self.connection.raw_put(URL_ADMIN_FLOWS_EXECUTIONS.format(**params_path),
-                                            data=payload)
+                                           data=payload)
         return raise_error_from_response(data_raw, KeycloakGetError, expected_code=204)
 
     def sync_users(self, storage_id, action):
